/****
 *
 * Exemplo: Menus.c
 *
 * Captulo: 3
 * Seo: 3.8
 *
 * Demonstra: Interao dirigida por menus.
 *
 ****/

#include <stdio.h>
#include <stdarg.h>

#define TITULO_OPCAO1 "Soma de dois numeros inteiros"
#define TITULO_OPCAO2 "Multiplicacao de dois nmeros inteiros"
#define TITULO_OPCAO3 "Divisao de dois numeros inteiros"
#define TITULO_OPCAO4 "Saida do programa"

#define N_OPCOES       4
#define OPCAO1 '1'
#define OPCAO2 '2'
#define OPCAO3 '3'
#define OPCAO4 '4'

/****
 *
 * Funo: LimpaBuffer
 *
 * Descrio: L e descarta caracteres encontrados na entrada padro
 *
 * Entrada: Nada
 * Sada: Nada
 * Retorno: Nada
 *
 ****/
void LimpaBuffer(void)
{
   int valorLido; /* valorLido deve ser int! */

   do {
      valorLido = getchar();
   } while ((valorLido != '\n') && (valorLido != EOF));
}

/****
 *
 * Funo: LeOpcao
 *
 * Descrio: L e valida a opo digitada pelo usurio
 *
 * Entrada: menorValor - o menor valor vlido
 *          maiorValor - o maior valor vlido
 * Sada: Nada
 * Retorno: A opo lida e validada
 *
 ****/
char LeOpcao(int menorValor, int maiorValor)
{
   int op;

   while (1) {
      printf("\nDigite sua opcao: ");
      op =  getchar();

      if (op >= menorValor && op <= maiorValor) {
         LimpaBuffer();
         break;
      } else {
         printf("\nOpcao invalida. Tente novamente.");
         printf("\nA opcao deve estar entre %c e %c.\n", menorValor, maiorValor);
         LimpaBuffer();
      }
   }

   return op;
}

/****
 *
 * Funo: LeInteiro
 *
 * Descrio: L um nmero inteiro introduzido pelo usurio
 *
 * Entrada: Nada
 * Sada: Nada
 * Retorno: O inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int  umInt, nValoresLidos;

   printf("Digite um valor inteiro: ");
   nValoresLidos = scanf("%d", &umInt);

   while (nValoresLidos == 0) { // Nenhum inteiro foi lido ainda
      LimpaBuffer();
      printf("Entrada incorreta. Digite um valor inteiro: ");
      nValoresLidos = scanf("%d", &umInt);
   }

   return umInt;
}

/****
 *
 * Funo: ApresentaMenu
 *
 * Descrio: Apresenta um menu com um nmero indeterminado de opes
 *
 * Entrada: nItens - o nmero de itens no menu
 *          menorOpcao - o caractere associado ao primeiro item
 * Sada: Nada
 * Retorno: Nada
 *
 ****/
void ApresentaMenu(int nItens, int menorOpcao, ...)
{
   int i;
   va_list 	argumentos;

   va_start(argumentos, menorOpcao); 	/* Inicializa lista de argumentos variveis */

			/* L cada argumento e imprime */
   for (i = 0; i < nItens; ++i) {
      printf("%c-%s\n", menorOpcao++, va_arg(argumentos, char *));
   }

   va_end(argumentos);	/* Encerra o processamento dos argumentos */
}

int main(void)
{
   unsigned char op;
   int           inteiro1, inteiro2;
   unsigned int  saida = 0;

   do {
      ApresentaMenu( N_OPCOES, OPCAO1,
                     TITULO_OPCAO1, TITULO_OPCAO2, TITULO_OPCAO3, TITULO_OPCAO4 );
      op = LeOpcao(OPCAO1, OPCAO1 + N_OPCOES - 1);

      switch(op) {
         case OPCAO1:
            inteiro1 = LeInteiro();
            inteiro2 = LeInteiro();
            printf("%d + %d = %d\n", inteiro1, inteiro2, inteiro1 + inteiro2);
            break;
         case OPCAO2:
            inteiro1 = LeInteiro();
            inteiro2 = LeInteiro();
            printf("%d * %d = %d\n", inteiro1, inteiro2, inteiro1 * inteiro2);
            break;
         case OPCAO3:
            inteiro1 = LeInteiro();
            inteiro2 = LeInteiro();
            printf("%d + %d = %d\n", inteiro1, inteiro2, inteiro1 + inteiro2);
            break;
         case OPCAO4:
            saida = 1;
            printf("Obrigado por usar este programa. Bye.");
            break;
         default:
            printf("Este programa possui um bug");
            return 1;
      }
   } while (!saida);

   return 0;
}
