/****
 *
 * Ttulo: Leitura de strings
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 03/08/2002
 * ltima modificao: 04/02/2007
 *
 * Descrio Geral: Este programa demonstra a leitura de strings usando
 *					LeString() apresentada na Seo 8.8.
 *
 * Entrada: Strings do tamanho especificado
 *
 * Sada: Quando o string introduzido pelo usurio  vlido, este string 
 *			 impresso. Quando o string introduzido  invlido, i.e., de tamnho
 *			 maior do que permitido, imprime uma mensagem de erro indicando a
 *			 causa do erro.
 *
 ****/

#include <stdio.h>
#include <string.h> /* Contm aluso de strlen()  */
#include <ctype.h>	/* Contm aluso de isdigit() */

#define MAX_NOME  5       /* Nmero mximo de caracteres no nome do usurio */
#define MAX_ENDERECO  10  /* Nmero mximo de caracteres no endereo do usurio */
#define CARACTERES_TELEFONE  7  /* Nmero exato de caracteres no */
                                /* nmero de telefone do usurio */

#define TAMANHO_BUFFER  20  /* Tamanho do arranjo a ser utilizado na */
                            /* leitura de strings                    */

	/* Tipo de ponteiro para funes de validao de strings */
typedef unsigned (*tFuncValidaString)(const char*);

/************   Aluses   ************/

extern char *LeString(char *, unsigned);
extern char *LeEValidaString( char *, unsigned, const char*,
									   const char*, tFuncValidaString );
extern unsigned ValidaNome(const char *);
extern unsigned ValidaTelefone(const char *);
extern void LimpaBuffer(void);

/****
 *
 * A funo main() demonstra a leitura e validao de strings.
 * Esta demonstrao  dividida em duas partes:
 *		(1) Na primeira parte,  aceito qualquer string que respeite o numero
 *			 mximo de caracteres permitido e encerra apenas quando um string
 * 		     dentro desse limite for lido.
 *		(2) A segunda parte do programa l strings representando o nome, endereo
 *			 e telefone do usurio com as seguintes restries:
 *         	* Nome: mximo de %d caracteres e no pode ser vazio.
 *         	* Endereo: mximo de %d caracteres, mas pode ser vazio.
 *         	* Telefone: exatamente %d caracteres.
 *          Apenas strings vlidos so impressos.
 *
 *
 ****/

int main(void)
{
   char bufferDeEntrada[TAMANHO_BUFFER];    /* Armazena os strings lidos */
   char *stringLido;  /* Apontar para o string lido se este for vlido */

   const char *promptNome = "\nDigite seu nome: ";
   const char *promptEndereco = "\nDigite seu endereco: ";
   const char *promptTelefone = "\nDigite seu telefone: ";

   const char *erroNome = "\nNome invalido.";
   const char *erroEndereco = "\nEndereco invalido." ;
   const char *erroTelefone = "\nTelefone invalido.";

   printf("** Este programa demonstra uma tecnica de leitura e validacao de strings **\n\n");

   printf("\nEsta parte do programa aceita qualquer string que respeite o numero"
   		 "\nmaximo de caracteres permitido e encerra apenas quando um string"
          "\ndentro desse limite for lido. \n\n");

   printf("\nIntroduza um string com no maximo %d caracteres: ", TAMANHO_BUFFER - 1);
   stringLido = LeString(bufferDeEntrada, TAMANHO_BUFFER);  /* Tenta ler um string */

   while(1) {  /* Este lao termina com break no seu interior */
      if (stringLido) {   /* Valor lido est ok? */
         printf("\nO string lido foi: %s\n", stringLido);
         break;    /* Tchau while */
      }
      printf("\nO string introduzido contem mais de %d caracteres: ", TAMANHO_BUFFER - 1);
      printf("\nIntroduza um string com no maximo %d caracteres: ", TAMANHO_BUFFER - 1);
      stringLido = LeString(bufferDeEntrada, TAMANHO_BUFFER);  /* Tenta novamente */
   }

   printf("\n**************************************************************\n");
   printf("\nEsta parte do programa le strings representando o nome, endereco e "
   		  " telefone do usuario com as seguintes restricoes: \n"
          "\tNome: maximo de %d caracteres e nao pode ser vazio.\n"
          "\tEndereco: maximo de %d caracteres, mas pode ser vazio.\n"
          "\tTelefone: exatamente %d caracteres.\n"
          "Apenas strings validos sao impressos.\n",
          MAX_NOME, MAX_ENDERECO, CARACTERES_TELEFONE);

   stringLido = LeEValidaString(bufferDeEntrada, MAX_NOME+1, promptNome,
   										erroNome, ValidaNome);
   printf("\nO nome introduzido foi: %s", stringLido ? stringLido : "INVALIDO");

   stringLido = LeEValidaString(bufferDeEntrada, MAX_ENDERECO+1, promptEndereco,
   										erroEndereco, (tFuncValidaString) 0);
   printf("\nO endereco introduzido foi: %s", stringLido ? stringLido : "INVALIDO");

   stringLido = LeEValidaString(bufferDeEntrada, CARACTERES_TELEFONE+1, promptTelefone,
   										erroTelefone, ValidaTelefone);
   printf("\nO telefone introduzido foi: %s", stringLido ? stringLido : "INVALIDO");

   printf("\n\n**************************************************************\n");
   printf("\nPrograma encerrado. Bye, bye.");

   return 0;
}

/****
 *
 * A funo LeString() l um string no meio de entrada contendo um nmero
 * mximo de caracteres especificado
 *
 * Argumentos: Sada: buffer - arranjo de caracteres que conter o string lido
 *			   Entrada: nElementos - tamanho do arranjo buffer
 *
 * Retorno: ponteiro para o string lido ou NULL, caso contrrio.
 *
 ****/

char *LeString(char * buffer, unsigned nElementos)
{
   unsigned contador = 0;   /* Conta o nmero de caracteres lidos */
   char     caracterLido;

   		/* L no mximo nElementos - 1 caracteres ou at encontrar '\n' */
   caracterLido = getchar();
   while ((caracterLido != '\n') && (contador < nElementos-1)) {
      buffer[contador++] = caracterLido;
      caracterLido = getchar();
   }

   buffer[contador] = '\0'; /* Termina o string */

   /* Se o ltimo caractere lido for '\n', o usurio digitou menos caracteres */
   /* do que o mximo permitido. Se o prximo caractere no stream de entrada  */
   /* for '\n', ele digitou exatamente o mximo permitido. Nestes dois casos, */
   /* tudo ocorreu bem e resta apenas retornar o endereo do arranjo buffer.  */
   /* Mas, se nem o ltimo caractere lido nem prximo caractere no stream for */
   /* '\n', o usurio digitou caracteres alm do permitido. Neste caso,       */
   /* limpa-se o buffer do stream de entrada e retorna-se 0.	  					*/

   if ((caracterLido == '\n') || (getchar() == '\n'))  /* Tudo bem */
      return buffer;
   else {      /* Usurio digitou caracteres demais */
      LimpaBuffer(); /* Limpa a sujeira deixada pelo usurio */
      return (char *) 0;
   }
}

 /****
 *
 * A funo LimpaBuffer() l e descarta todos os caracteres que porventura
 * tenham sido deixados no buffer de entrada numa tentativa de leitura de dados
 *
 * Argumentos: nenhum
 *
 * Valor de retorno: nenhum
 *
 * NOTA: Voc no precisa modificar esta funo nem deve utilizar fflush()
 * para descarregar o buffer de entrada. A funo fflush() serve apenas
 * para bufffers de sada.
 *
 ****/

void LimpaBuffer(void)
{
   int valorLido = getchar(); /* valorLido deve ser int! */

        /* L caracteres no buffer de entrada enquanto    */
        /* no econtra final de linha ou final de arquivo */
   while ((valorLido != '\n') && (valorLido != EOF))
      valorLido = getchar();
}

/****
 *
 * A funo LeEValidaString() l um string no meio de entrada contendo um nmero
 * mximo de caracteres especificado e que satisfaz um critrio especificado
 *
 * Argumentos: Sada: buffer - arranjo de caracteres que conter o string lido
 *			   Entrada: nElementos - tamanho do arranjo buffer
 *			            prompt - a indio de entrada apresentada ao usurio
 *			            erro - a mensagem de erro apresentada ao usurio
 *			            valida - ponteiro para uma funo de validao do
 *			                     string; se este argumento for NULL, no existe
 *			                     critrio de validao.
 *
 * Retorno: ponteiro para o string lido e validado ou NULL, caso contrrio.
 *
 ****/

char *LeEValidaString(char * buffer, unsigned nElementos, const char* prompt,
									   const char* erro, tFuncValidaString valida)
{
   char *stringLido;

   printf("%s", prompt); /* Jamais faa isto: printf(prompt); pois, se prompt */
                         /* contiver o caracter '%', voc estar perdido      */

   /* Tenta ler o string usando LeString(). Se o string retornado por esta */
   /* funo for NULL, imprime uma mensagem de erro e retorna NULL. Caso   */
   /* contrrio, se for fornecida uma funo de validao e o string no   */
   /* satisfaz o critrio especificado nesta funo, imprime uma mensagem  */
   /* de erro e retorna NULL. Em qualquer outra situao, simplesmente     */
   /* retorna o string lido.																*/

   if (!(stringLido = LeString(buffer, nElementos))) {
      printf("%s", erro); /* Jamais faa isto: printf(erro); */;
      return stringLido; /* Neste caso, stringLido  NULL */
   } else if (valida && !valida(stringLido)) { /* A funo de validao existe */
      printf("%s", erro); 						  /* e o critrio no foi satisfeito */
      return (char *) 0;
   }

   return stringLido;
}

/****
 *
 * A funo ValidaNome() verifica se o argumento recebido satisfaz
 * critrios de validade do nome.
 *
 * Argumento (entrada): oNome - o string a ser verificado.
 *
 * Retorno: 1, se o nome no  vazio; 0, em caso contrrio.
 *
 ****/

unsigned ValidaNome(const char *oNome)
{
	return (strlen(oNome) > 0);
}

/****
 *
 * A funo ValidaTelefone() verifica se o argumento recebido satisfaz
 * critrios de validade do endereco.
 *
 * Argumento (entrada): oEndereco - o string a ser verificado.
 *
 * Retorno: 1, se o telefone contm apenas dgitos e na quantidade especificada;
 *          0, em caso contrrio.
 *
 ****/

unsigned ValidaTelefone(const char *oTelefone)
{
   if (strlen(oTelefone) != CARACTERES_TELEFONE) /* O nmero de caracteres deve */
      return 0;								 /* ser exatamente igual ao estabelecido */

   while (*oTelefone)
      if (!isdigit(*oTelefone++))  /* Checa se cada caractere  dgito */
         return 0;  /* Algum caractere no era dgito */

   return 1; /* Se no retornou ainda, est tudo OK */
}

