/****
 *
 * Implementao do mdulo Leitura2  (Seo 10.9)
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 04/07/2004
 * ltima modificao: 22/08/2004
 *
 * Descrio Geral: Prov funes para leitura e validao de dados.
 *
 ****/

#include <stddef.h>
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Leitura2.h"

#define TAMANHO_DO_ARRANJO  50 /* Tamanho do arranjo que armazenar o contedo  */
                               /* do buffer. Esta constante  necessria apenas */
                               /* neste arquivo, por isso ela  definida aqui   */
                               /* e no no	cabealho do mdulo.                */

 /***************************************************************************/
 /*  Funes LOCAIS a este arquivo. Estas funes servem apenas como		*/
 /*  auxlio na implementao do mdulo. Portanto, elas no so exportadas. */
 /***************************************************************************/

/****
 *
 * A funo ImprimeMensagemDeErro() apresenta mensagens de erro no stream padro
 * de erros.
 *
 * Argumentos (entrada): formato - string de formao do mesmo tipo usado por printf()
 *								 ... - representa aquilo que ser impresso usando formato
 *
 * Retorno: aquilo que  retornado pela funo vfprintf() (consulte o Help).
 *
 * NOTA: Voc no precisa modificar esta funo.
 *
 ****/

static unsigned ImprimeMensagemDeErro(const char *formato, ...)
{
   va_list argumentos;
   int     resultado;

   va_start (argumentos, formato); /* Faz argumentos apontar para o primeiro     */
                                   /* argumento da lista de argumentos variveis */

      /* Chama a funo vfprintf() para imprimir a lista de argumentos variveis      */
      /* utilizando formato como string de formatao. A impresso  feita em stderr. */
      /* Por isso, no  necessrio usar fflushc().                                   */
   resultado = vfprintf(stderr, formato, argumentos);

   va_end (argumentos);  /* Encerra o processamento da lista de argumentos variveis */

   return resultado;
}

/****
 *
 * A funo IndicaErro() indica a posio aonde um erro ocorre
 * na entrada de dados do usurio.
 *
 * Argumentos (entrada): str - string que representa a entrada de dados
 *								 ptrErro - ponteiro para o primeiro caracter que
 *											  causa o erro.
 * Retorno: Nada
 *
 * NOTA: Voc no precisa modificar esta funo.
 *
 ****/

static void IndicaErro(const char *str, const char *ptrErro)
{
                  /* Esta varivel aponta para a mensagem introdutria: */
   static const char *const mensagem = "Entrada ilegal. Foi encontrado um "
                                       "erro na posicao indicada abaixo:\n";

   if (errno != 0) {  /* Foi detectado um erro na converso */
      ImprimeMensagemDeErro("%s\n", strerror(errno)); /* Imprime mensagem do sistem */
      ImprimeMensagemDeErro(mensagem);  /* Imprime parte inicial da indicao de erro */
   } else
      ImprimeMensagemDeErro(mensagem);  /* Idem */

   ImprimeMensagemDeErro("    %s\n", str);  /* Imprime o string lido na entrada */

          /* Imprime o caractere '^' na posio do erro */
   ImprimeMensagemDeErro("    %*c\n", (int) (ptrErro - str) + 1, '^');
}

 /***************************************************************************/
 /* 							   *** Funes Globais ***			        */
 /***************************************************************************/

/****
 *
 * A funo LeValor() l um string no meio de entrada padro, converte-o
 * para o tipo double e verifica se o valor convertido satisfaz os
 * critrios especificados
 *
 * Argumentos: Sada: valor - ponteiro para double que apontar para o
 *									   valor lido em caso de sucesso. Caso a funo
 *			                           no seja bem sucedida, *valor  indefinido
 *					Entrada: pfVerifica - ponteiro para uma funo de verificao,
 *												 que recebe um valor double como entrada e
 *												 retorna 1 se o valor lido satisfaz os
 *												 critrios de validade e 0, em caso contrrio
 *
 * Retorno: 1, se bem sucedida ou 0, caso contrrio.
 *
 ****/

unsigned LeValor(double *valor, unsigned (*pfVerifica)(double))
{
   size_t nCaracteresLidos; /* O nmero de caracteres lido na entrada padro */
   char   strEntrada[TAMANHO_DO_ARRANJO]; /* Arranjo que conter o string lido */

   char   *ptrFinal; /* Este ponteiro ir apontar para o caractere do string */
                      /* que causar o final da tentativa de converso         */

   while (1) {
      printf("Introduza um numero de ponto flutuante: ");
      fflush(stdout); /* Garante que o usurio l o prompt imediatamente */

    					/* L a entrada e armazena-a no arranjo strEntrada */
      if (!fgets(strEntrada, sizeof(strEntrada), stdin))
         return 0;  /* O stream de entrada est vazio. */
                    /* No h nada mais a ser feito.   */

      nCaracteresLidos = strlen(strEntrada); /* O comprimento do string no     */
                                                /* arranjo strEntrada corresponde */
                                                /* ao nmero de caracteres lidos  */

    	   /* Quando a funo fgets() encontra '\n' antes de retornar, ela inclui    */
    	   /* este caractere como penltimo caractere do string lido. Este caractere */
    	   /* precisa ser removido antes de tentar fazer a converso do string.      */

      if (strEntrada[nCaracteresLidos-1] == '\n') { /* Testa se '\n' foi lido */
         strEntrada[--nCaracteresLidos] = '\0'; /* Remove o caracter '\n' e subtrai */
                                                /* 1 do nmero de caracteres lidos  */

              /*  necessrio inicializar a varivel errno com zero, pois assim  */
              /* pode-se garantir que, se o valor desta varivel for != 0 aps a */
              /* chamada de strtod(), a causa deste valor ter sido esta funo. */
         errno = 0;

         *valor = strtod(strEntrada, &ptrFinal);  /* Tenta converter o string */

                /* Se nenhum erro for detectado na converso, pode-se sair  */
         		/* do while. Caso contrrio, chama-se a funo que imprime  */
         		/* uma mensagem correspondente ao erro encontrado. 			*/
         if (!errno && nCaracteresLidos && !*ptrFinal) /* Nenhum erro detectado */
            break; 												 /* Sai do while */

         IndicaErro(strEntrada, ptrFinal);

      } else {  /* No foi encontrado o caracter '\n'. Isto significa */
    			/* que o usurio digitou caracters demais.			 */
         LimpaBuffer(); /* Limpa o buffer de entrada que contm pelo menos '\n' */
      				    /* Informa ao usurio que a entrada foi grande demais:  */
         ImprimeMensagemDeErro("A entrada foi muito grande. O numero maximo "
         				       "de caracteres e' %d.\n", TAMANHO_DO_ARRANJO);
      } /* if */
   } /* while */

  		/* O valor lido foi realmente do tipo especificado (double). Resta */
  		/* determinar se este valor satisfaz outros critrios especficos  */

   if (!pfVerifica) /* Se o ponteiro para funo passado como parmetro  nulo,    */
      return 1;	    /* nada mais precisa ser validado; i.e., o valor lido est OK. */

   if (pfVerifica(*valor))  /* Utiliza o ponteiro para funo passado como */
      return 1;			    /* parmetro para chamar a funo de validao */
   else {
      ImprimeMensagemDeErro("O valor %lf nao satisfaz os criterios do programa.\n", *valor);
      return 0;
   }
}

/****
 *
 * A funo LimpaBuffer() l e descarta todos os caracteres que porventura
 * tenham sido deixados no buffer de entrada numa tentativa de leitura de dados
 *
 * Argumentos: nenhum
 *
 * Valor de retorno: nenhum
 *
 * NOTA: Voc no precisa modificar esta funo nem deve utilizar fflush()
 * para descarregar o buffer de entrada. A funo fflush() serve apenas
 * para bufffers de sada.
 *
 ****/

void LimpaBuffer(void)
{
   int valorLido = getchar(); /* valorLido deve ser int! */

         /* L caracteres no buffer de entrada enquanto    */
         /* no econtra final de linha ou final de arquivo */
   while ((valorLido != '\n') && (valorLido != EOF))
      valorLido = getchar();
}



