/****
 *
 * Exemplo: Bubble3.c
 *
 * Captulo: 11
 * Seo: 11.9.2
 *
 * Demonstra: Generalizao da implementao do mtodo
 *            da bolha usando alocao dinmica de memria.
 *
 ****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  FALSE  0
#define  TRUE   1

int ComparaIntsCrescente(const void *elem1, const void *elem2)
{
   return *(int *)elem1 - *(int *)elem2;
}

int ComparaIntsDecrescente(const void *elem1, const void *elem2)
{
   return *(int *)elem2 - *(int *)elem1;
}

int CompStrings(const void *e1, const void *e2)
{
   return(strcmp((char *)e1,(char *)e2));
}

void BubbleSort3( void *lista, size_t nElementos,
                  size_t tamElemento,  /* Alterao 1 */
                  int (*compara) (const void *, const void *))
{
   unsigned  i, ordenada = FALSE;
   char 	 *aux;  /* Alterao 2 */

   aux = malloc(tamElemento); /* Alterao 4 */

   if (!aux)    /* Alterao 5 */
      return;

   while (!ordenada){
      ordenada = TRUE;	/* Supe que a lista est ordenada */

      for (i = 0; i < nElementos - 1; i++){
         if (compara(lista + i*tamElemento,
                     lista + (i + 1)*tamElemento) > 0){/*Alterao 6*/
            ordenada = FALSE; /*Pelo menos um elemento fora de ordem*/

			   /* Troca elementos adjacentes */
			   /* Alteraes 6, 7 e 8 */
            memmove(aux, lista + i*tamElemento, tamElemento);
            memmove(lista + i*tamElemento,
                     lista + (i + 1)*tamElemento, tamElemento);
            memmove(lista + (i + 1)*tamElemento, aux, tamElemento);
         }  /* final do if */
      } /* final do for */
   } /* final do while */
} /* final da funo BubbleSort3 */

int main(void)
{
   int  arrayDeInts[] = {12, 55, 21, 1, 6, 8, 17, 220, 5, 83};
   char cores[5][8] = { "azul", "verde", "preto", "roxo", "branco" };
   int  i, tamanhoAr;

   tamanhoAr = sizeof(arrayDeInts)/sizeof(arrayDeInts[0]);

   printf("Ordem original:\n");
   for (i=0; i < tamanhoAr; i++) {
     printf(" %d ", arrayDeInts[i]);
   }

   BubbleSort3(arrayDeInts, tamanhoAr, sizeof(int), ComparaIntsCrescente);

   printf("\n\nOrdem crescente:\n");
   for (i=0; i < tamanhoAr; i++) {
     printf(" %d ", arrayDeInts[i]);
   }

   BubbleSort3(arrayDeInts, tamanhoAr, sizeof(int), ComparaIntsDecrescente);

   printf("\n\nOrdem decrescente:\n");
   for (i=0; i < tamanhoAr; i++) {
     printf(" %d ", arrayDeInts[i]);
   }

   BubbleSort3((void *)cores, 5, sizeof(cores[0]), CompStrings);

   printf("\n\nOrdenacao de strings:\n");
   for (i = 0; i < 5; i++)
      printf("%s\n", cores[i]);

   return 0;
}

