/****
 *
 * Exemplo: ArquivosClass.c
 *
 * Captulo: 12
 * Seo: 12.11
 *
 * Demonstra: Acesso direto:
 *            Classificao externa de arquivos por indexao.
 *
 ****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  NUMERO_MAXIMO_DE_REGISTROS   200
#define  COMPRIMENTO_DO_NOME   	       20

typedef  struct  {
            short  dia, mes, ano;
         } tData;

typedef  struct  {
            char  nome[COMPRIMENTO_DO_NOME];
            tData  data;
         } tRegistro;

typedef  struct {
            int   indice;
            char  chave[COMPRIMENTO_DO_NOME];
         } tPares;


void LimpaBuffer(void)
{
   int valorLido = getchar(); /* valorLido deve ser int! */

   while ((valorLido != '\n') && (valorLido != EOF))
   valorLido = getchar();
}

char *LeString(char * buffer, unsigned nElementos)
{
   unsigned contador = 0;   /* Conta o nmero de caracteres lidos */
   char     caracterLido;

   		/* L no mximo nElementos - 1 caracteres ou at encontrar '\n' */
   caracterLido = (char) getchar();
   while ((caracterLido != '\n') && (contador < nElementos-1)) {
      buffer[contador++] = caracterLido;
      caracterLido = (char) getchar();
   }

   buffer[contador] = '\0'; /* Termina o string */

   if ((caracterLido == '\n') || (getchar() == '\n'))  /* Tudo bem */
      return buffer;
   else {      /* Usurio digitou caracteres demais */
      LimpaBuffer(); /* Limpa a sujeira deixada pelo usurio */
      return (char *) 0;
   }
}

/****
 *
 * Funo ColetaPares(): L registros de um arquivo e armazena a chave
 *                       de cada registro juntamente com seu ndice.
 *
 * Parmetros:
 *      stream (entrada): stream onde ser feita a leitura
 *      arrayDePares (sada): array que armazenar os pares chave/ndice
 *      maximoDeRegistros (entrada): nmero mximo de registro lidos
 *
 * Retorno: nmero de registros lidos, se no houver erro;
 *          zero, em caso contrrio.
 *
 ****/

unsigned ColetaPares( FILE *stream, tPares arrayDePares[],
                      unsigned  maximoDeRegistros )
{
   unsigned   i, nItenLidos, distancia = 0;

   for (i = 0; i < maximoDeRegistros; i++) {
              /* L o campo nome de cada registro e armazena */
              /* no campo chave do array de pares            */
      nItenLidos = fread( arrayDePares[i].chave, 1,
                          COMPRIMENTO_DO_NOME, stream );

      if (nItenLidos < COMPRIMENTO_DO_NOME) /* O final de arquivo */
         break;                        /* foi atingido ou ocorreu */
                                       /* algum erro de leitura   */

          /* Armazena ndice do registro lido*/
      arrayDePares[i].indice = i;

          /* Posiciona apontador de arquivo para prxima leitura */
      distancia += sizeof(tRegistro);
      if (fseek(stream, distancia, SEEK_SET) && !feof(stream))
         return 0; /* No foi possvel mover o apontador de posio */
   } /* for */

   return  i;
}

/****
 *
 * Funo ClassificaPares(): Classifica um array de elementos do tipo
 *                           tPares utilizando a funo qsort().
 *
 * Parmetros:
 *      arrayDePares (entrada/sada): array que armazena pares
 *                                    chave/ndice
 *      numeroDeElementos (entrada): nmero elementos do array
 *
 * Retorno: Nada.
 *
 ****/

void   ClassificaPares( tPares  arrayDePares[],
                        unsigned  numeroDeElementos)
{
       /* Aluso da funo que ser utilizada com a funo qsort() */
   int Compara(const void *, const void *);

   qsort(arrayDePares, numeroDeElementos, sizeof(tPares), Compara);
}

/****
 *
 * Funo Compara(): Funo de comparao utilizada por qsort().
 *                   Utiliza a funo strcmp() para comparar os
 *                   campos "nome" dos registros.
 *
 * Parmetros:
 *      p1 (entrada): ponteiro para um par do tipo tPares
 *      p2 (entrada): ponteiro para outro par do tipo tPares
 *
 * Retorno: 0, se as chaves so iguais
 *          > 0, se a primeira chave  maior do que a segunda
 *          < 0, se a primeira chave  menor do que a segunda
 *
 ****/

int Compara(const void *p1, const void *p2)
{
   return strcmp(((tPares *)p1)->chave, ((tPares *)p2)->chave);
}

/****
 *
 * Funo ImprimeRegistrosClassificados(): Imprime registros ordenados
 *
 * Parmetros:
 *      stream (entrada): stream onde ser feita a leitura
 *      arrayDePares (entrada): array que armazena os pares
 *                              chave/ndice ordenados pela chave
 *      numeroDeElementos (entrada): nmero de elementos do array
 *
 * Retorno: nmero de registros lidos, se no houver erro;
 *          zero, em caso contrrio.
 *
 ****/

unsigned ImprimeRegistrosClassificados( FILE  *stream,
                                        tPares  arrayDePares[],
                                        unsigned numeroDeElementos )
{
   tRegistro   umRegistro;
   unsigned    i;

   for (i = 0;  i < numeroDeElementos; i++) {
      if ( fseek(stream, sizeof(tRegistro)*arrayDePares[i].indice,
           SEEK_SET) )
         return 0; /* No foi possvel mover apontador de arquivo */

            /* L o registro especificado  */
      fread(&umRegistro, sizeof(tRegistro), 1, stream);
            /* Imprime o registro recuperado */
      printf("%s \t\t %d/%d/%d\n", umRegistro.nome,
              umRegistro.data.dia,
              umRegistro.data.mes, umRegistro.data.ano);
   }

   return 1;
}

/****
 *
 * main()
 *
 ****/

int main(int  argc, char  *argv[])
{
   FILE  	*stream;
   tPares	pares[NUMERO_MAXIMO_DE_REGISTROS];
   unsigned	numeroDeRegistrosLidos;

         /* Deveria haver dois nomes na execuo deste  */
         /* programa: os nomes do programa e do arquivo */
   if (argc != 2){
      printf("Erro: nome do arquivo ausente\n");
      return 1;
   }

   stream = fopen(argv[1], "rb");
   if (!stream) {       /* Arquivo especificado na linha de comando */
	printf("Erro abrindo arquivo %s\n", argv[1]); /* no pode ser  */
	return 1;				                      /*   aberto      */
   }

   numeroDeRegistrosLidos = ColetaPares( stream, pares,
                                         NUMERO_MAXIMO_DE_REGISTROS );
   if ( numeroDeRegistrosLidos) {
      ClassificaPares(pares, numeroDeRegistrosLidos);
      if ( !ImprimeRegistrosClassificados(stream, pares,
                                           numeroDeRegistrosLidos) ) {
         printf("Erro processando arquivo.\n"); /* Houve algum erro */
         fclose(stream);
         return 1;
      }
   }

   fclose(stream);

   return 0;
}
