/****
 *
 * Exemplo: ArquivosCopia.c
 *
 * Captulo: 12
 * Seo: 12.8
 *
 * Demonstra: Processamento seqencial de arquivos.
 *
 ****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  TAMANHO_DA_LINHA   100
#define  TAMANHO_DO_BLOCO   512

typedef  char  tDados;

/****
 *
 * Funo CopiaArquivoPorCaractere(): Copia um arquivo byte a byte.
 *
 * Parmetros:
 *      arquivoDeEntrada (entrada): nome do arquivo que ser copiado
 *      arquivoDeSaida (entrada): nome do arquivo que receber a cpia
 *
 * Retorno: 1, se no houver erro; zero, em caso contrrio.
 *
 ****/

unsigned CopiaArquivoPorCaractere( const char  *arquivoDeEntrada,
                                   const char  *arquivoDeSaida )
{
   FILE  *ptrEntrada, *ptrSaida;   /* Streams de entrada e sada */
   char  c;

   ptrEntrada = fopen(arquivoDeEntrada, "rb");

   if (ptrEntrada == NULL)
      return 0;	/* Arquivo de entrada no pode ser aberto */

      /* Neste ponto o arquivo de entrada foi aberto com sucesso    */
      /* Se o arquivo de sada no puder ser aberto, deve-se fechar */
      /* o arquivo de entrada antes de retornar			        */
   if ((ptrSaida = fopen(arquivoDeSaida, "wb")) == NULL) {
      fclose(ptrEntrada);  /* Arquivo de sada no pode ser aberto */
      return 0;
   }

   while (1) {
      c = getc(ptrEntrada);
      if (feof(ptrEntrada)) /* Testa se final do arquivo */
         break;  		    /* de entrada foi atingido   */
      putc(c, ptrSaida);
   }

       /* Processamento terminado.  necessrio fechar os arquivos */
   fclose(ptrEntrada);
   fclose(ptrSaida);

   return 1;
}


/****
 *
 * Funo CopiaArquivoPorLinha(): Copia um arquivo linha a linha.
 *
 * Parmetros:
 *      arquivoDeEntrada (entrada): nome do arquivo que ser copiado
 *      arquivoDeSaida (entrada): nome do arquivo que receber a cpia
 *
 * Retorno: 1, se no houver erro; zero, em caso contrrio.
 *
 ****/

unsigned  CopiaArquivoPorLinha( const char  *arquivoDeEntrada,
                                const char  *arquivoDeSaida )
{
   FILE  *ptrEntrada, *ptrSaida;    /* Streams de entrada e sada  */
   char   linha[TAMANHO_DA_LINHA];  /* Array de caracteres onde os */
                              /* caracteres lidos sero armazenados */

   ptrEntrada = fopen(arquivoDeEntrada, "rb");

   if (ptrEntrada == NULL)
      return 0;  /* Arquivo de entrada no pode ser aberto */

    /* Neste ponto o arquivo de entrada foi aberto com sucesso. */
    /* Se o arquivo de sada no puder ser aberto, deve-se      */
    /* fechar o arquivo de entrada antes de retornar.           */

   ptrSaida = fopen(arquivoDeSaida, "wb");

   if (ptrSaida == NULL) {
      fclose(ptrEntrada);  /* Arquivo de sada no pode ser aberto */
      return 0;
   }

   while ( fgets(linha, TAMANHO_DA_LINHA, ptrEntrada) != NULL )
      fputs(linha, ptrSaida);

        /* Processamento terminado.  necessrio fechar os arquivos */
   fclose(ptrEntrada);
   fclose(ptrSaida);

   return 1;
}

/****
 *
 * Funo CopiaArquivoPorBloco(): Copia um arquivo bloco a bloco.
 *
 * Parmetros:
 *      arquivoDeEntrada (entrada): nome do arquivo que ser copiado
 *      arquivoDeSaida (entrada): nome do arquivo que receber a cpia
 *
 * Retorno: 1, se no houver erro; zero, em caso contrrio.
 *
 ****/

unsigned  CopiaArquivoPorBloco( const char  *arquivoDeEntrada,
                                const char  *arquivoDeSaida )
{
   FILE    *ptrEntrada, *ptrSaida; /* Streams de entrada e sada */
   tDados bloco[TAMANHO_DO_BLOCO]; /* Array de caracteres onde     */
                          /* os caracteres lidos sero armazenados */
   unsigned  numeroDeItensLidos;

   ptrEntrada = fopen(arquivoDeEntrada, "rb");

   if (!ptrEntrada)
      return 0;  /* Arquivo de entrada no pode ser aberto */

   ptrSaida = fopen(arquivoDeSaida, "wb");

    /* Neste ponto o arquivo de entrada foi aberto com sucesso. */
    /* Se o arquivo de sada no puder ser aberto, deve-se      */
    /* fechar o arquivo de entrada antes de retornar.           */

   if (!ptrSaida) { /* Arquivo de sada no pode ser aberto */
      fclose(ptrEntrada); /* Fecha arquivo de entrada... */
      return 0;           /* antes de retornar           */
   }

   do {
      numeroDeItensLidos = fread( bloco, sizeof(tDados),
                                  TAMANHO_DO_BLOCO, ptrEntrada );
      fwrite(bloco, sizeof(tDados), numeroDeItensLidos, ptrSaida);
   } while ( numeroDeItensLidos == TAMANHO_DO_BLOCO );

   fclose(ptrEntrada);
   fclose(ptrSaida);

    /* A funo fread() retorna 0 quando no encontra    */
    /* mais  elementos ou quando ocorre um erro antes da */
    /* leitura de qualquer elemento. A funo ferror()  */
    /* usada para resolver esta ambigidade.             */

   if (ferror(ptrEntrada))   /* Verifica se houve erro de leitura */
      return 0;

   return 1;
}


void LimpaBuffer(void)
{
   int valorLido = getchar(); /* valorLido deve ser int! */

   while ((valorLido != '\n') && (valorLido != EOF))
      valorLido = getchar();
}

char *LeString(char * buffer, unsigned nElementos)
{
   unsigned contador = 0;   /* Conta o nmero de caracteres lidos */
   char     caracterLido;

   		/* L no mximo nElementos - 1 caracteres ou at encontrar '\n' */
   caracterLido = (char) getchar();
   while ((caracterLido != '\n') && (contador < nElementos-1)) {
      buffer[contador++] = caracterLido;
      caracterLido = (char) getchar();
   }

   buffer[contador] = '\0'; /* Termina o string */

   if ((caracterLido == '\n') || (getchar() == '\n'))  /* Tudo bem */
      return buffer;
   else {      /* Usurio digitou caracteres demais */
      LimpaBuffer(); /* Limpa a sujeira deixada pelo usurio */
      return (char *) 0;
   }
}

/****
 *
 * main()
 *
 ****/

int main(void)
{
   char        strArq[20], strCopia[30];
   const char *promptStr = "\nIntroduza o nome do arquivo a ser copiado: ";
   int         op;

   printf(promptStr);
   if (!LeString(strArq, 20) || !*strArq)
      return 0;

   printf( "\nOpcoes disponiveis:\n\t\t1 = copia por caractere\n\t\t2 = copia por linha"
   		   "\n\t\t3 = copia por bloco\n\t\toutro valor = sai do programa"
           "\n\nEscolha sua opcao: " );
   op = getchar();

   switch(op) {
      case '1':
         strcpy(strCopia, "Caract");
         strcat(strCopia, strArq);
         if (CopiaArquivoPorCaractere(strArq, strCopia))
            printf("\nCopia bem sucedida. Bye.");
         else
            printf("\nA copia falhou. Sorry.");
         break;
      case '2':
         strcpy(strCopia, "Linha");
         strcat(strCopia, strArq);
         if (CopiaArquivoPorLinha(strArq, strCopia))
            printf("\nCopia bem sucedida. Bye.");
         else
            printf("\nA copia falhou. Sorry.");
         break;
      case '3':
         strcpy(strCopia, "Bloco");
         strcat(strCopia, strArq);
         if (CopiaArquivoPorBloco(strArq, strCopia))
            printf("\nCopia bem sucedida. Bye.");
         else
            printf("\nA copia falhou. Sorry.");
         break;
   }

   return 0;
}

